!----------------------------------------------------------
!   resolution de l'equation de convection lineaire 2d 
!      (sous forme de loi de conservation d/dt f + div(v f)=0
!   - a champ de vitesse variable
!   - sur grille cartesienne
!   - domaine rectangulaire defini par : 
!             NORD
!            _______ y=+h
!            |     |
!    OUEST   |     |       EST
!            _______ y=-h
!           x=0    x=L
!              SUD
!   - schema volumes finis, decentre d'ordre 1, euler explicite en temps
!   - fichiers de donnees : 
!         1) geometrie.data (contient 1 seule ligne : L,h)
!         2) maillaga.data (contient 1 seule ligne : imax,jmax)
!----------------------------------------------------------

program conv2d_cart

  !----------- sous programmes ------------------------------------ 
  use mod_precision   !- (double ou simple precision)
  use mod_vitesse     !- (champ de vitesse)
  use mod_ci          !- (donnee initiale)
  use mod_aplusmoins  !- (calcul de a+ et a-)
  use mod_sorties     !- (ecriture fichier de resultats)
  use mod_sorties     !- resultats de calculs
  !----------------------------------------------------------------

  implicit none

  !------------ variables -----------------------------------------
  integer :: i,j,imax,jmax,nmax,n,k,pas_affichage
  character(len=100) :: numero
  real(pr) :: L,h,dx,dy,cfl,invdt,dt,vmax,tmax
  real(pr), dimension(:), allocatable :: x,y,xc,yc
  real(pr), dimension(:,:), allocatable :: f,fnpun 
  !----------------------------------------------------------------


  !------------- programme ----------------------------------------

  !--- parametres geometriques
  open(unit=10,file='geometrie.data')
  read(10,*) L,h
  close(10)

  !--- parametres de maillage
  open(unit=20,file='maillage.data')
  read(20,*) imax,jmax
  close(20)

  !--- calcul des coordonnees des noeuds et des centres de maille
  allocate(x(0:imax),y(0:jmax),xc(1:imax),yc(1:jmax))
  dx=L/imax
  x=(/(i*dx,i=0,imax)/)
  xc=(/((i-0.5_pr)*dx,i=1,imax)/)
  dy=2*h/jmax
  yc=(/(-h+(j-0.5_pr)*dy,j=1,jmax)/)
  y=(/(-h+j*dy,j=0,jmax)/)

  !--- pas de temps
  cfl=0.9_pr
  invdt=0._pr 
  vmax=0._pr
  do i=0,imax
     do j=0,jmax
        invdt=max(invdt,abs(vx(x(i),y(j)))/dx+abs(vy(x(i),y(j))/dy))
        vmax=max(vmax,sqrt((vx(x(i),y(j)))**2+(vy(x(i),y(j)))**2))
     end do
  end do
  dt=cfl/invdt

  !--- donne initiale : pour changer de donnee, modifier le module mod_ci
  allocate(f(0:imax+1,0:jmax+1),fnpun(1:imax,1:jmax))
  f(1:imax,1:jmax)=ci(xc,yc,L)

  !--- conditions aux limites : on met 0 partout
  f(0,:)=0._pr      !- ouest
  f(imax+1,:)=0._pr !- est
  f(:,0)=0._pr      !- sud
  f(:,jmax+1)=0._pr !- nord

  !--- temps de simulation et parametres de sorties

  !-- exemple 1 : profil parabolique
  tmax=L/vmax !- (pour aller d'un bout a l'autre du domaine)

!!$  !-- exemple 2 : rotation rigide
!!$  tmax=2*acos(-1.0_pr) !- (pour faire un tour complet)

  nmax=floor(tmax/dt) 
  print*,"nombre d'iterations en temps :",nmax
  pas_affichage=max(nmax/120,1)  !-- pour avoir 120 images, soit 5 secondes d'animation
  k=0
  

  !--- evolution
  do n=1,nmax

     !-- iteration en temps
     do i=1,imax
        do j=1,jmax
           fnpun(i,j)=f(i,j)                                                                              &
                &           -dt/dx*(  (aplus(vx(x(i),yc(j)))*f(i,j)+amoins(vx(x(i),yc(j)))*f(i+1,j))      &
                &                    -(aplus(vx(x(i-1),yc(j)))*f(i-1,j)+amoins(vx(x(i-1),yc(j)))*f(i,j))  &
                &                  )                                                                      &
                &           -dt/dy*(  (aplus(vy(xc(i),y(j)))*f(i,j)+amoins(vy(xc(i),y(j)))*f(i,j+1))      &
                &                    -(aplus(vy(xc(i),y(j-1)))*f(i,j-1)+amoins(vy(xc(i),y(j-1)))*f(i,j))  &
                &                  )                                                                             
        end do
     end do

     !-- mise a jour
     f(1:imax,1:jmax)=fnpun

     !-- sauvegarde resultats (pour medit ou visit)
     if (mod(n,pas_affichage)==0) then

        k=k+1
        write(numero,*) k
        if (k<=9) then
           numero='00'//trim(adjustl(numero))
        elseif (9<k.and.k<=99) then
           numero='0'//trim(adjustl(numero))
        end if

!!$        call sortie_medit(numero,imax,jmax,f,x,y)
        call sortie_visit(numero,imax,jmax,f,x,y)
        
        print*,n

     end if

  end do

  !----------------------------------------------------------------

end program conv2d_cart
